import 'dart:async';
import 'dart:math';

import 'package:flutter/material.dart';
import 'package:get/get.dart';

class WidgetBackgroundGradientRunningBuilder extends StatefulWidget {
  final Widget child;
  final List<Color> colors;
  final Duration duration;

  final EdgeInsets? margin;
  final EdgeInsets? padding;
  final BorderRadius? borderRadius;

  const WidgetBackgroundGradientRunningBuilder({
    Key? key,
    required this.colors,
    required this.child,
    this.duration = const Duration(seconds: 5),
    this.padding,
    this.margin,
    this.borderRadius,
  }) : super(key: key);

  @override
  State<WidgetBackgroundGradientRunningBuilder> createState() =>
      _WidgetBackgroundGradientRunningBuilderState();
}

class _WidgetBackgroundGradientRunningBuilderState
    extends State<WidgetBackgroundGradientRunningBuilder> {
  final Random rd = Random();
  final ValueNotifier<Alignment> valueNotifier =
      ValueNotifier(const Alignment(0.0, -.5));
  Timer? _timer;
  @override
  void initState() {
    super.initState();
    _timer = Timer.periodic(widget.duration, (timer) {
      double x = rd.nextBool() ? -rd.nextDouble() : rd.nextDouble();
      double y = rd.nextBool() ? -rd.nextDouble() : rd.nextDouble();
      valueNotifier.value = Alignment(x, y);
    });
  }

  @override
  void dispose() {
    super.dispose();
    _timer?.cancel();
    _timer = null;
    valueNotifier.dispose();
  }

  @override
  Widget build(BuildContext context) {
    return ValueListenableBuilder<Alignment>(
      valueListenable: valueNotifier,
      builder: (context, value, child) {
        return AnimatedContainer(
          duration: widget.duration,
          width: Get.width,
          height: Get.height,
          margin: widget.margin,
          padding: widget.padding,
          decoration: BoxDecoration(
            borderRadius: widget.borderRadius,
            gradient: RadialGradient(
              focalRadius: -0.35,
              radius: 0.85,
              center: value,
              colors: widget.colors,
            ),
          ),
          child: child,
        );
      },
      child: widget.child,
    );
  }
}
